# Generated by Django 2.1.12 on 2020-01-21 12:27

import django.db.models.deletion
import gm2m.fields
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('apis_metainfo', '0001_initial'),
        ('contenttypes', '0002_remove_content_type_name'),
        ('apis_highlighter', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name='annotation',
            name='entity_candidate',
            field=models.ManyToManyField(blank=True, to='apis_metainfo.UriCandidate'),
        ),
        migrations.AddField(
            model_name='annotation',
            name='entity_link',
            field=gm2m.fields.GM2MField('apis_entities.Person', 'apis_entities.Institution', 'apis_entities.Place', 'apis_entities.Event', 'apis_entities.Work', 'apis_relations.PersonPerson', 'apis_relations.PersonPlace', 'apis_relations.PersonInstitution', 'apis_relations.PersonEvent', 'apis_relations.PersonWork', 'apis_relations.InstitutionPlace', 'apis_relations.InstitutionEvent', 'apis_relations.InstitutionWork', 'apis_relations.InstitutionInstitution', 'apis_relations.PlaceEvent', 'apis_relations.PlaceWork', 'apis_relations.PlacePlace', 'apis_relations.EventWork', 'apis_relations.EventEvent', 'apis_relations.WorkWork', through_fields=('gm2m_src', 'gm2m_tgt', 'gm2m_ct', 'gm2m_pk')),
        ),
        migrations.AddField(
            model_name='annotation',
            name='parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='parent_annotation', to='apis_highlighter.Annotation'),
        ),
        migrations.AddField(
            model_name='annotation',
            name='text',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='apis_metainfo.Text'),
        ),
        migrations.AddField(
            model_name='annotation',
            name='user_added',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL),
        ),
    ]



import re

import reversion
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.db import models
from gm2m import GM2MField
from crum import get_current_request
from django.urls import reverse


##############################################
#
#   Project classes
#
##############################################


class Project(models.Model):
    """Holds information on registered project.
    Id is used by the JavaScript function to target the right endpoints
    """

    name = models.CharField(max_length=255)  # name of the project registered
    user = models.ForeignKey(
        User, on_delete=models.CASCADE
    )  # foreignkey to the User who created the project
    description = models.TextField(blank=True, null=True)
    base_url = models.URLField(
        blank=True, null=True
    )  # optional base URL to restrict highlights to a base URL
    store_text = models.BooleanField(
        default=False
    )  # Whether to store the text in the tool or work with ids only.

    def __str__(self):
        return self.name

    class Meta:
        db_table = "highlighter_project"


class TextHigh(models.Model):
    """Holds unstructured text associated with
    one ore many entities/relations.
    """

    title = models.CharField(max_length=255, blank=True, null=True)
    text_choices = (("ft", "Full Text"), ("id", "Text ID"), ("cl", "Text Class"))
    text_type = models.CharField(max_length=3, choices=text_choices, default="cl")
    uri = models.URLField(
        blank=True, null=True
    )  # uri of vocab used to identify the text type
    text = models.TextField(blank=True, null=True)
    text_id = models.PositiveIntegerField(
        blank=True, null=True
    )  # UID to identify the text within the project. Allows to not upload the texts.
    text_class = models.CharField(max_length=255, blank=True, null=True)
    project = models.ForeignKey(
        Project, blank=True, null=True, on_delete=models.SET_NULL
    )

    def __str__(self):
        return self.title

    class Meta:
        db_table = "highlighter_texthigh"


class AnnotationProject(models.Model):
    """Every Project can have several Annotation Projects. Annotation Projects are used to track the precission etc.
    of automatic annotation tools.
    """

    name = models.CharField(max_length=255)
    description = models.TextField(blank=True, null=True)
    published = models.BooleanField(default=False)

    @classmethod
    def from_db(cls, db, field_names, values):
        instance = super().from_db(db, field_names, values)
        instance._loaded_values = dict(zip(field_names, values))
        return instance

    def save(self, *args, **kwargs):
        if hasattr(self, "_loaded_values"):
            if self.published != self._loaded_values["published"]:
                for ann in self.annotation_set.all():
                    for ent in ann.entity_link.all():
                        if ent.published != self.published:
                            ent.published = self.published
                            ent.save()
        super().save(*args, **kwargs)

    def __str__(self):
        return self.name

    class Meta:
        db_table = "highlighter_annotationproject"


@reversion.register()
class Annotation(models.Model):
    """Class storing highlights in full-texts"""

    set_highlighter = getattr(
        settings,
        "APIS_HIGHLIGHTER_ENTITIES",
        (
            "apis_entities.Person",
            "apis_entities.Institution",
            "apis_entities.Place",
            "apis_entities.Event",
            "apis_entities.Work",
            "apis_relations.PersonPerson",
            "apis_relations.PersonPlace",
            "apis_relations.PersonInstitution",
            "apis_relations.PersonEvent",
            "apis_relations.PersonWork",
            "apis_relations.InstitutionPlace",
            "apis_relations.InstitutionEvent",
            "apis_relations.InstitutionWork",
            "apis_relations.InstitutionInstitution",
            "apis_relations.PlaceEvent",
            "apis_relations.PlaceWork",
            "apis_relations.PlacePlace",
            "apis_relations.EventWork",
            "apis_relations.EventEvent",
            "apis_relations.WorkWork",
        ),
    )
    status_choices = (("del", "deleted"), ("ap", "approved"))
    start = models.PositiveIntegerField()  # number of string to start highlight
    end = models.PositiveIntegerField()  # number of string to end highlight
    entity_link = GM2MField(
        *set_highlighter
    )  # generic field to store the relation object
    entity_candidate = models.ManyToManyField("apis_metainfo.UriCandidate", blank=True)
    orig_string = models.CharField(
        max_length=255, blank=True, null=True
    )  # string originally highlighted
    text = models.ForeignKey("apis_metainfo.Text", on_delete=models.CASCADE)
    parent = models.ForeignKey(
        "self",
        related_name="parent_annotation",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
    )  # parent annotations are used to allow for a stacked design of the annotation project
    user_added = models.ForeignKey(
        User, blank=True, null=True, on_delete=models.SET_NULL
    )  # changed from default=12
    annotation_project = models.ForeignKey(
        AnnotationProject, blank=True, null=True, on_delete=models.CASCADE
    )
    status = models.CharField(
        max_length=4, choices=status_choices, blank=True, null=True
    )

    def __str__(self):
        return "{}".format(self.pk)

    def trim_whitespaces(self, char):
        test = True
        while test and char < self.text_length:
            if self.text.text[char] == " ":
                test = True
                char += 1
            else:
                return char

    def annotation_hash(self, format_string="start_end_text_ent_entid"):
        """
        Function that returns a hash of the annotaion used to calculate inter-annotator agreement.

        :return:
        """
        matching = {"start": "start", "end": "end", "text": "text_id"}
        f_list = format_string.split("_")
        if "start" not in f_list or "end" not in f_list or "text" not in f_list:
            raise AttributeError(
                "Start, end and text must be present in the format_string parameter"
            )
        res = ""
        self.text_length = len(self.text.text)
        self.start = self.trim_whitespaces(self.start)
        self.end = self.trim_whitespaces(self.end)
        for f in f_list:
            if f in matching.keys():
                res += str(getattr(self, matching[f]))
            else:
                ent_link = self.entity_link.all()
                if len(ent_link) > 0:
                    cont_lst = str(
                        ContentType.objects.get_for_model(ent_link[0])
                    ).split()
                    if f == "ent":
                        res += "".join(cont_lst)
                    elif f == "entid":
                        for cont in cont_lst:
                            if hasattr(ent_link[0], "related_" + cont.strip()):
                                res += str(
                                    getattr(
                                        ent_link[0], "related_" + cont.strip() + "_id"
                                    )
                                )
                            elif hasattr(ent_link[0], "related_" + cont.strip() + "A"):
                                res += str(
                                    getattr(
                                        ent_link[0], "related_" + cont.strip() + "A_id"
                                    )
                                )
                            elif hasattr(ent_link[0], "related_" + cont.strip() + "B"):
                                res += str(
                                    getattr(
                                        ent_link[0], "related_" + cont.strip() + "B_id"
                                    )
                                )
                            res += "-"
                        res = res[:-1]
                    elif f == "rel":
                        res += str(getattr(ent_link[0], "relation_type_id"))
                else:
                    res += "NONE"
            res += "_"
        return res[:-1]

    def get_related_entity(self):
        rel = self.entity_link.all()
        if not len(rel) == 1:
            return None
        else:
            return rel[0]

    def get_html_markup(self, include_object=False):
        if not len(self.entity_link.all()) == 1:
            return None
        rel_entity = self.entity_link.all()[0]
        if hasattr(rel_entity, "relation_type"):
            entity_kind = str(rel_entity.relation_type.pk)
        elif hasattr(rel_entity, "kind"):
            entity_kind = str(rel_entity.kind.pk)
        else:
            entity_kind = str(0)
        entity_type = type(rel_entity).__name__
        entity_type_app = ContentType.objects.get(model=entity_type.lower()).app_label
        ent_lst_pk = []
        if self.user_added is not None:
            user_added = self.user_added.username[:2]
        else:
            user_added = "na"
        if entity_type_app.lower() == "apis_relations":
            for x in dir(rel_entity):
                c = re.match("related_\w+_id", x)
                if c:
                    ent_lst_pk.append(str(getattr(rel_entity, c.group(0))))
        start_span = """<mark class="highlight hl_text_{}" data-hl-type="simple" data-hl-start="{}"
            data-hl-end="{}" data-hl-text-id="{}" data-hl-ann-id="{}" data-entity-class="{}" data-entity-pk="{}"
            data-related-entity-pk="{}" data-entity-type="{}" data-user-added={}>""".format(
            entity_kind,
            self.start,
            self.end,
            self.text_id,
            self.pk,
            entity_type,
            rel_entity.pk,
            ",".join(ent_lst_pk),
            entity_type_app,
            user_added,
        )
        start_span = start_span.replace("\n", "")
        if include_object:
            request = get_current_request()
            if request is None:
                base = getattr(settings, "APIS_BASE_URI", "http://to_change.com")
                if base.endswith("/"):
                    base = base[:-1]
                url = f'{base}{reverse("apis:apis_api:{}-detail".format(entity_type.lower()), kwargs={"pk": rel_entity.pk},)}'
            else:
                url = request.build_absolute_uri(
                    reverse(
                        "apis:apis_api:{}-detail".format(entity_type.lower()),
                        kwargs={"pk": rel_entity.pk},
                    )
                )
            res_obj = {
                "id": self.id,
                "start": self.start,
                "end": self.end,
                "related_object": {
                    "id": rel_entity.id,
                    "url": url,
                    "type": entity_type,
                    "label": str(rel_entity),
                },
            }
            return start_span, res_obj
        return start_span

    class Meta:
        db_table = "highlighter_annotation"


##############################################
#
#   Configuration Classes
#
##############################################


class VocabularyAPI(models.Model):
    """Class storing the information of the endpoints of the vocabularies."""

    method_lst = (("l", "local"), ("o", "open skos"))
    name = models.CharField(max_length=255, null=True, blank=True)
    api_endpoint = models.CharField(max_length=255, null=True, blank=True)
    method = models.CharField(max_length=2, choices=method_lst, default="l")

    def __str__(self):
        return self.name

    class Meta:
        db_table = "highlighter_vocabularyapi"


class MenuEntry(models.Model):
    """Class that defines the menu structure of the context menu"""

    choices_kind = (
        ("txt", "Text field"),
        ("frm", "Form"),
        ("m", "menu entry"),
        ("fn", "Javascript function"),
    )
    kind = models.CharField(max_length=4, choices=choices_kind, default="txt")
    name = models.CharField(
        max_length=255, null=True, blank=True
    )  # Charfield used when txt is as type of entry specified
    api = models.ForeignKey(
        VocabularyAPI, blank=True, null=True, on_delete=models.SET_NULL
    )  # Foreignkey used to link the API entry in case api is set as kind
    parent = models.ForeignKey(
        "self",
        related_name="parent_menuEntry",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
    )
    project = models.ForeignKey(
        Project, null=True, blank=True, on_delete=models.CASCADE
    )

    def __str__(self):
        return self.name

    class Meta:
        db_table = "highlighter_menuentry"


#######################################################################
#
# DL-Classes
#
#######################################################################


class ActiveLearningProject(models.Model):
    """Used to store active learning project settings."""

    choices = ""
    name = models.CharField(max_length=255)
    sampling_strategy = models.CharField(max_length=4, choices=choices)
    rebuild = models.IntegerField()
    collection = models
    log_dir = models.CharField(max_length=255)

    class Meta:
        db_table = "highlighter_activelearningproject"



from django.test import TestCase
from django.urls import reverse
from rest_framework.authtoken.models import Token
from rest_framework.test import APIClient
from apis_core.apis_metainfo.models import Text
from apis_core.apis_entities.models import Person, Institution, Event
from django.contrib.auth.models import User
from .models import Annotation
from rest_framework import status
from random import randint


class HighlighterAPITestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        # Set up data for the whole TestCase
        user = User.objects.create_user(username="lauren", password="pas_1234$")
        token = Token.objects.create(user=user)
        client = APIClient()
        client.credentials(HTTP_AUTHORIZATION="Token " + token.key)
        cls.c = client
        txt = "Franz fuhr mit dem Rad nach Wien."
        start = 6
        end = 10
        start_1 = 19
        end_1 = 22
        start_2 = 28
        end_2 = 32
        print(txt[start:end])
        cls.text = Text.objects.create(text=txt)
        cls.text_original = txt
        cls.annotation = Annotation.objects.create(text=cls.text, start=start, end=end)
        cls.annotation_2 = Annotation.objects.create(
            text=cls.text, start=start_1, end=end_1
        )
        cls.annotation_3 = Annotation.objects.create(
            text=cls.text, start=start_2, end=end_2
        )
        pers = Person.objects.first()
        cls.annotation.entity_link.add(pers)
        inst = Institution.objects.first()
        cls.annotation_2.entity_link.add(inst)
        ev = Event.objects.first()
        cls.annotation_3.entity_link.add(ev)
        print(cls.annotation)

    def test_get(self):
        url = (
            reverse(f"apis:apis_core:text-detail", kwargs={"pk": self.text.pk})
            + "?format=json&highlight"
        )
        res = self.c.get(url)
        print(res)
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        elements = res.json()
        if ">fuhr</mark>" not in elements["text"]:
            raise ValueError("Highlight not correct")
        print(self.annotation.start, self.annotation.end, self.text_original)
        self.assertEqual(
            self.text_original[self.annotation.start : self.annotation.end], "fuhr"
        )
        self.assertEqual(
            self.text_original[self.annotation_2.start : self.annotation_2.end], "Rad"
        )
        self.assertEqual(
            self.text_original[self.annotation_3.start : self.annotation_3.end], "Wien"
        )
        print(elements)

    def test_add_newlines(self):
        txt = "Franz\n fuhr mit\n\n dem Rad nach\n Wien."
        self.text.text = txt
        self.text.save()
        for an in Annotation.objects.filter(text=self.text):
            if self.text.text[an.start:an.end] not in ["fuhr", "Rad", "Wien"]:
                print(f"value pure annotation: {self.text.text[an.start:an.end]}")
                raise ValueError("Annotation not correctly moved")
            else:
                print(f"moved annotation: {self.text.text[an.start:an.end]}")

        url = (
            reverse(f"apis:apis_core:text-detail", kwargs={"pk": self.text.pk})
            + "?format=json&highlight"
        )
        res = self.c.get(url)
        print(res)
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        elements = res.json()
        print(f"text section of return: {elements}")
        if ">fuhr</mark>" not in elements["text"]:
            raise ValueError("Highlight not correct")
        if ">Rad</mark>" not in elements["text"]:
            raise ValueError("second Highlight not correct")
        if ">Wien</mark>" not in elements["text"]:
            raise ValueError("third Highlight not correct")

    def test_hard(self):

        def randomize_and_compare(text_object, mutation_iteration):

            class Fragment():

                def __init__(self, text=None, is_annotated=None):
                    self.text = text
                    self.is_annotated = is_annotated


            def create_fragment_list(text_str, anns_ordered):
                # create list of subtext, differentiating between annotated and non-annotated fragments

                s = 0
                e = 0
                ann_current = None
                fragment_list = []
                while e < len(text_str) - 1:
                    if ann_current is None:
                        if len(anns_ordered) > 0:
                            ann_current = anns_ordered[0]
                            del anns_ordered[0]
                            e = ann_current.start
                            fragment_list.append(
                                Fragment(
                                    text=text_str[s:e],
                                    is_annotated=False,
                                )
                            )
                            s = ann_current.start
                        else:
                            e = len(text_str)
                            fragment_list.append(
                                Fragment(
                                    text=text_str[s:e],
                                    is_annotated=False,
                                )
                            )
                    else:
                        if len(anns_ordered) > 0:
                            ann_next = anns_ordered[0]
                            if ann_current.end < ann_next.start:
                                e = ann_current.end
                                fragment_list.append(
                                    Fragment(
                                        text=text_str[s:e],
                                        is_annotated=True,
                                    )
                                )
                                s = ann_current.end
                                ann_current = None
                            else:
                                ann_current = ann_next
                                del anns_ordered[0]
                        else:
                            e = ann_current.end
                            fragment_list.append(
                                Fragment(
                                    text=text_str[s:e],
                                    is_annotated=True,
                                )
                            )
                            s = ann_current.end
                            ann_current = None

                return fragment_list


            def randomize_texts(fragment_list, mutation_iteration):

                def get_random_pos(s):

                    return randint(0, len(s) - 1)

                def chance_50():

                    if randint(0,1) == 1:
                        return True
                    else:
                        return False

                def chance_66():

                    if randint(0, 2) < 2:
                        return True
                    else:
                        return False

                def randomize_string(t):

                    if chance_50(): # 50% chance for randomizing this text
                        s = get_random_pos(t) # random start index for subtext
                        e = get_random_pos(t) # random end index for subtext
                        if s > e:
                            tmp = s
                            s = e
                            e = tmp
                        diff = e - s
                        s += round(diff / 4) # minimizing the distance between subtext-indices to make subtext smaller
                        e -= round(diff / 4)
                        if chance_66(): # 66% chance for adding subtext
                            if chance_50(): # 50% chance for adding subtext to end
                                t = t + t[s:e]
                            else: # 50% chance for adding subtext to front
                                t = t[s:e] + t
                        else: # 33% chance for removing subtext
                            t = t[:s] + t[e:]
                        if chance_50():
                            i = get_random_pos(t)
                            t = t[:i] + "\n" + t[i:]

                    return t

                for i in range(mutation_iteration):
                    for fragment in fragment_list:
                        if not fragment.is_annotated:
                            fragment.text = randomize_string(fragment.text)

                return "".join([f.text for f in fragment_list])

            def main(text_object, mutation_iteration):

                anns_ordered = list(text_object.annotation_set.all().order_by("start"))
                text_original = text_object.text
                anns_to_compare = {}
                for ann in anns_ordered:
                    anns_to_compare[ann.pk] = {
                        "old": text_original[ann.start:ann.end],
                        "new": None,
                        "correct": None
                    }
                fragment_list = create_fragment_list(text_original, anns_ordered)
                text_object.text = randomize_texts(fragment_list, mutation_iteration)
                text_object.save()
                for ann in text_object.annotation_set.all():
                    anns_to_compare[ann.pk]["new"] = text_object.text[ann.start:ann.end]
                    anns_to_compare[ann.pk]["correct"] = anns_to_compare[ann.pk]["old"] == anns_to_compare[ann.pk]["new"]
                for k in anns_to_compare.keys():
                    d = anns_to_compare[k]
                    if d["correct"] == True:
                        print(f"id: {k} - correct")
                    elif d["correct"] == False:
                        print(f"id: {k} - ERROR: Wrong text")
                        is_all_good = False
                    elif d["correct"] is None:
                        print(f"id: {k} - ERROR: Annotation was deleted")
                        is_all_good = False

            main(text_object, mutation_iteration)

        print("---------------------------------------------------------")
        print("Starting random mutation of non-annotated text fragments")

        for mutation_iteration in range(0, 6):

            text_object = Text.objects.create(
                text=r"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Aliquam gravida dui eget erat laoreet, a porttitor nisl fermentum. Sed quis metus sed risus tincidunt blandit. Aliquam eu ultricies ipsum. Sed quis vulputate libero, eget hendrerit ligula. Suspendisse sed diam purus. Ut pretium mattis nisl, nec posuere urna porta eu. Donec sit amet risus leo. Suspendisse risus sapien, tristique eget pulvinar condimentum, luctus eu dui. Maecenas maximus vulputate elit at suscipit. Nunc ac suscipit nisi, ac pretium eros. Mauris eget aliquam metus. Morbi ut euismod dolor, vitae ultrices orci. Maecenas vulputate placerat libero eu porta. Morbi rutrum ex a ipsum rutrum aliquet. Donec quis neque efficitur, auctor enim sed, sagittis lorem. Aliquam dictum lacus a tempus pulvinar. \nProin in neque imperdiet tellus iaculis venenatis fermentum in sem. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Curabitur tincidunt a elit quis ultrices. Nullam pulvinar a ligula quis blandit. In nec lacus eu risus imperdiet pharetra. Proin vel elit at ipsum gravida faucibus non sit amet lectus. Duis pulvinar vestibulum interdum. Nullam ullamcorper risus sit amet nisi elementum mollis. Aenean non enim libero. \nInteger condimentum dolor sed dolor pulvinar, sed lacinia nisi tempor. Quisque turpis lorem, convallis ac egestas a, condimentum a enim. Praesent eget mauris a quam fringilla porta eu eget urna. Integer tristique urna leo, in auctor ante faucibus eget. Aliquam erat volutpat. Nunc turpis ex, hendrerit sed elit dignissim, rhoncus consectetur tortor. Praesent semper eros et eros tristique scelerisque. Nulla erat enim, aliquam vitae commodo vel, vehicula eu justo. Sed a venenatis nulla, vitae bibendum risus. Proin sed libero tincidunt mauris varius volutpat in at lectus. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Etiam vel lacus ut justo malesuada pharetra vitae in tellus. Donec ullamcorper sit amet ipsum in consequat. Mauris a ex cursus magna placerat convallis. Nam a lobortis ipsum. Nunc a vehicula mauris. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Aliquam gravida dui eget erat laoreet, a porttitor nisl fermentum. Sed quis metus sed risus tincidunt blandit. Aliquam eu ultricies ipsum. Sed quis vulputate libero, eget hendrerit ligula. Suspendisse sed diam purus. Ut pretium mattis nisl, nec posuere urna porta eu. Donec sit amet risus leo. Suspendisse risus sapien, tristique eget pulvinar condimentum, luctus eu dui. Maecenas maximus vulputate elit at suscipit. Nunc ac suscipit nisi, ac pretium eros. Mauris eget aliquam metus. Morbi ut euismod dolor, vitae ultrices orci. Maecenas vulputate placerat libero eu porta. Morbi rutrum ex a ipsum rutrum aliquet. Donec quis neque efficitur, auctor enim sed, sagittis lorem. Aliquam dictum lacus a tempus pulvinar. \nProin in neque imperdiet tellus iaculis venenatis fermentum in sem. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Curabitur tincidunt a elit quis ultrices. Nullam pulvinar a ligula quis blandit. In nec lacus eu risus imperdiet pharetra. Proin vel elit at ipsum gravida faucibus non sit amet lectus. Duis pulvinar vestibulum interdum. Nullam ullamcorper risus sit amet nisi elementum mollis. Aenean non enim libero. \nInteger condimentum dolor sed dolor pulvinar, sed lacinia nisi tempor. Quisque turpis lorem, convallis ac egestas a, condimentum a enim. Praesent eget mauris a quam fringilla porta eu eget urna. Integer tristique urna leo, in auctor ante faucibus eget. Aliquam erat volutpat. Nunc turpis ex, hendrerit sed elit dignissim, rhoncus consectetur tortor. Praesent semper eros et eros tristique scelerisque. Nulla erat enim, aliquam vitae commodo vel, vehicula eu justo. Sed a venenatis nulla, vitae bibendum risus. Proin sed libero tincidunt mauris varius volutpat in at lectus. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Etiam vel lacus ut justo malesuada pharetra vitae in tellus. Donec ullamcorper sit amet ipsum in consequat. Mauris a ex cursus magna placerat convallis. Nam a lobortis ipsum. Nunc a vehicula mauris."
            )
            Annotation.objects.create(text=text_object, start=981, end=990)
            Annotation.objects.create(text=text_object, start=2575, end=2583)
            Annotation.objects.create(text=text_object, start=105, end=132)
            Annotation.objects.create(text=text_object, start=673, end=703)
            Annotation.objects.create(text=text_object, start=715, end=780)
            Annotation.objects.create(text=text_object, start=1297, end=1368)
            Annotation.objects.create(text=text_object, start=1829, end=1875)
            Annotation.objects.create(text=text_object, start=1987, end=2046)
            Annotation.objects.create(text=text_object, start=2257, end=2324)
            Annotation.objects.create(text=text_object, start=2382, end=2388)
            Annotation.objects.create(text=text_object, start=2392, end=2459)
            Annotation.objects.create(text=text_object, start=2748, end=2777)
            Annotation.objects.create(text=text_object, start=2815, end=2832)
            Annotation.objects.create(text=text_object, start=2845, end=2850)

            print(f"starting mutation iteration: {mutation_iteration}")

            randomize_and_compare(text_object, mutation_iteration)
